{
open Lexing
open Ast_program
open Parser_program

exception LexerError of string



let newline lexbuf =
  let pos = lexbuf.lex_curr_p in
  lexbuf.lex_curr_p <-
    { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum}
}

let alpha = ['_' 'a'-'z' 'A'-'Z' '0'-'9']
let ident = alpha+
let space = ' ' |'\t'
let comment = "//" [^'\n']*

rule token = parse
  | '\n' { newline lexbuf ; token lexbuf }
  | (space | comment)+ { token lexbuf }
  | ident as id { IDENT id }
  | '<'  { LSEND }
  | '>'  { RSEND }
  | '('  { LPAR  }
  | ')'  { RPAR  }
  | '.'  { DOT   }
  | '|'  { COMP  }
  | '*'  { EMPTY }
  | eof  { EOF }
  | _ as c { raise (LexerError ("Caractère non autorisé : "^ String.make 1 c)) }
                        
