{
open Lexing
open Parser_sugar


let newline lexbuf =
  let pos = lexbuf.lex_curr_p in
  lexbuf.lex_curr_p <-
    { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum}

}

let digit = ['0' - '9']
let alpha = ['a'-'z' 'A'-'Z']
let ident = '<' | '>' | ')' | '(' | '.' | '|' | '*' | alpha ('_' | alpha | digit)*
let space = ' ' |'\t'
let comment = "//" [^'\n']*

rule token = parse
  | '\n' { newline lexbuf ; IDENT "\n" }
  | space+ as c { IDENT c }
  | comment+ { token lexbuf }
  | "+1" { PLUSONE }
  | "++" { INCR }
  | digit+ as x { NUMBER (int_of_string x) }
  | "/*" { COMMENTLEFT  }
  | "*/" { COMMENTRIGHT }
  | '[' { LBRACKET }
  | ']' { RBRACKET }
  | ';' { SEMICOLON }
  | '!' { EXCLAM }
  | '?' { INTERRO }
  | "::" { CONS }
  | '{' { LBRACE }
  | '}' { RBRACE }
  | ident as id { IDENT id }
  | eof  { EOF }
  | _ as c { IDENT (Char.escaped c) }

