open Ast_program
open Parser_program
open Lexer_program
open Compiler_program

open Format
open Arg
open Lexing

let usage = "usage: prepare file.hoc"

let file =
  let f = ref None in
  let set_file s =
    if not (Filename.check_suffix s ".hoc")
    then raise (Arg.Bad "no .hoc extension");
    f := Some s in
  Arg.parse [] set_file usage;
  match !f with
  | None -> Arg.usage [] usage; exit 1
  | Some f -> f

let reportPos (b,e) =
  let l = b.pos_lnum in
  let fc = b.pos_cnum - b.pos_bol + 1 in
  let lc = e.pos_cnum - b.pos_bol + 1 in
  eprintf "File \"%s\", line %d, characters %d-%d:\n@." file l fc lc


let () =
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let f = Parser_program.init Lexer_program.token lb in
    close_in c;
    Compiler_program.compile file f
  with e -> close_in c; match e with
    | Lexer_program.LexerError s ->
      reportPos  (lexeme_start_p lb, lexeme_end_p lb);
      eprintf "Lexical error: %s@." s;
      exit 1
    | Parser_program.Error ->
      reportPos  (lexeme_start_p lb, lexeme_end_p lb);
      eprintf "Syntax error@.";
      exit 1
    | Compiler_program.Not_closed x ->
      eprintf "HOCore program should be a closed term (%s is undefined)@." x;
      exit 1
    | e ->
      eprintf "Anomaly: %s@." (Printexc.to_string e);
      exit 2
