%{

  open Lexing

  let rec num_to_succ = function
    | 0   -> "ZERO"
    | n -> "SUCC("^(num_to_succ (n-1))^")"

  let rec list_to_push n = function
    | a::l -> "Hd<"^a^"> | Len<"^(num_to_succ n)^"> | Tl<"
              ^(list_to_push (n-1) l)^">"
    | []   -> "EMPTY"

%}
     
%token LBRACKET RBRACKET SEMICOLON
%token LBRACE RBRACE
%token COMMENTLEFT COMMENTRIGHT
%token PLUSONE INCR
%token EXCLAM INTERRO CONS
%token EOF
%token <string> IDENT
%token <int> NUMBER

%start <string> init

%%

init:
| t = text_group; EOF { t }

text_group:
| t = text+
  { List.fold_right (fun t l -> t^l) t "" }

text:
| COMMENTLEFT comment COMMENTRIGHT
  { "" }
| LBRACKET l = separated_list(SEMICOLON, text_group) RBRACKET
  { list_to_push (List.length l) l }
| t = text; PLUSONE
  { "SUCC("^t^")"}
| LBRACE; v = text_group; RBRACE; CONS; c = IDENT
  { c^"(_).push("^v^",_) | RET_push(_)."^c^"<_>" }
| n = NUMBER
  { num_to_succ n }
| EXCLAM
  { "(_)._ |" }
| a = IDENT; INTERRO
  { "(_)._ | "^a^"<_> |" }
| a = IDENT; INCR
  { a^"(_)."^a^"<SUCC(_)>" }
| x = IDENT
  { if x = "_" then Format.printf "Spurious character: _\n"; x }

comment:
| comment_text+ { () }

comment_text:
| COMMENTLEFT comment COMMENTRIGHT
  { () }
| LBRACKET | EXCLAM | SEMICOLON | RBRACKET | PLUSONE | NUMBER | IDENT | CONS
  | INCR | LBRACE | RBRACE
  { () }
